class TouchstoneException(Exception):
    pass


class ServiceException(Exception):
    pass


class TestException(Exception):
    pass


class MockException(Exception):
    pass


class MockNotSupportedException(Exception):
    pass


class ContainerException(Exception):
    pass
